from aws_cdk import (
	App, Stack, Duration,
	aws_ec2 as ec2,
	aws_ecr as ecr,
	aws_ecs as ecs,
	aws_iam as iam,
	aws_codedeploy as codedeploy,
	aws_elasticloadbalancingv2 as elbv2,
	aws_certificatemanager as acm,
	aws_logs as logs
)
import aws_cdk as cdk


class P2uFrontendECSFarget(Stack):

	def __init__(self, scope: App, construct_id: str, environment: str, **kwargs) -> None:
		super().__init__(scope, construct_id, **kwargs)

		# Get Default VPC
		
		vpc = ec2.Vpc.from_lookup(self, "DefaultVpc", is_default=True)
		prefix = f'p2u-frontend-{environment}'
		self.cluster = ecs.Cluster(self, prefix, vpc=vpc)
		repository = None
		repository = "p2u-shop-frontend"

		# self.ecr_repository = ecr.Repository(self, "p2u-shop-backend")
		self.ecr_repository = ecr.Repository.from_repository_name(self, f'{prefix}-ecr', repository_name=repository)

		self.execution_role = None
		self.task_role = None
		self.service, self.log_group = self.create_ecs_service(vpc, prefix, self.ecr_repository.repository_uri, environment)
		self.alb, self.https_listener, self.target, self.target_1 = self.create_alb(self.service, vpc,  prefix, environment)
		self.deployment_group= self.create_code_deploy_deployment_group(self.service, self.alb, self.https_listener, self.target, self.target_1, prefix)



	def create_ecs_service(self, vpc: ec2.Vpc, prefix: str, repository_uri: str, environment: str):
		# Get the default security group
		default_security_group = ec2.SecurityGroup.from_lookup_by_name(
			self, "SecurityGroupLookup", "default", vpc)
		
		# Create a task execution role for the Fargate tasks
		self.execution_role = cdk.aws_iam.Role(self, f'{prefix}-fargate-execution-role',
											   assumed_by=cdk.aws_iam.ServicePrincipal("ecs-tasks.amazonaws.com"),
											   inline_policies={},
											   managed_policies=[
												   cdk.aws_iam.ManagedPolicy.from_aws_managed_policy_name(
													   "service-role/AmazonECSTaskExecutionRolePolicy")
											   ]
											   )
		# Create a task role
		self.task_role = None
		self.task_role = iam.Role(
			self, f'{prefix}-fargate-task-role',
			assumed_by=iam.ServicePrincipal('ecs-tasks.amazonaws.com'),
			description=f'ECS task role for p2u frontend',
			inline_policies={}
		)

		self.task_role.add_managed_policy(iam.ManagedPolicy.from_aws_managed_policy_name("AmazonSSMManagedInstanceCore"))
		self.task_role.add_managed_policy(iam.ManagedPolicy.from_aws_managed_policy_name("CloudWatchAgentServerPolicy"))

		# Create a Fargate service task definition
		fargate_task_definition = cdk.aws_ecs.FargateTaskDefinition(self, f'{prefix}-task-definition',
																	memory_limit_mib=2048,
																	cpu=1024,
																	execution_role=self.execution_role,
																	task_role=self.task_role,
																	)

		log_group = logs.LogGroup(
							self,
							"p2u-frotnend-log-group", 
							log_group_name=f"/aws/p2u-ecs-frontend-{environment}", 
							retention=logs.RetentionDays.INFINITE,
			)

		# Add a container to the task definition
		fargate_task_definition.add_container(prefix,
											  image=cdk.aws_ecs.ContainerImage.from_registry(
												  f'{repository_uri}:43fbfd9fa63a2971bf4f6713766a70773673bb74'),
											  health_check=ecs.HealthCheck(
												  command=[
													  "CMD-SHELL", "curl -f http://localhost:8000/ || exit 1"],
												  # Interval between health checks
												  interval=Duration.seconds(
													  30),
												  # Timeout for each health check
												  timeout=Duration.seconds(5)
											  ),
											  port_mappings=[ecs.PortMapping(
												  container_port=8000)],
											  logging=ecs.AwsLogDriver(
												  stream_prefix=prefix, 
												  log_group=log_group,
												  mode=ecs.AwsLogDriverMode.NON_BLOCKING),
												  memory_reservation_mib=1024
											  )
		# Create an ECS service
		fargate_service = cdk.aws_ecs.FargateService(
			self, "Service",
			service_name=prefix,
			cluster=self.cluster,
			security_groups=[default_security_group],
			task_definition=fargate_task_definition,
			desired_count=1,
			health_check_grace_period=Duration.seconds(300),
			deployment_controller=ecs.DeploymentController(type=ecs.DeploymentControllerType.CODE_DEPLOY),
			vpc_subnets=ec2.SubnetSelection(availability_zones=["ap-northeast-2a"]),
			assign_public_ip=True
		)

		min_capacity = None
		if environment == "prd":
			 min_capacity=2
		else:
			 min_capacity=1

		scalable_target = fargate_service.auto_scale_task_count(
			min_capacity=min_capacity,
			max_capacity=6
		)

		scalable_target.scale_on_cpu_utilization("CpuScaling",
			policy_name=f"{prefix}-scaling-policy",
			target_utilization_percent=50,
			scale_in_cooldown=Duration.seconds(60),  # Cooldown period for scaling in
			scale_out_cooldown=Duration.seconds(60),  # Cooldown period for scaling out
			disable_scale_in=False,  # Allow scaling in
		)


		return fargate_service, log_group

	def create_alb(self, service: cdk.aws_ecs.FargateService, vpc: ec2.Vpc, prefix: str, environment: str):

		alb = None
		certificate_arn = None
		certificate = None
		https_listener = None
		https_listener_1 = None
		https_listener_2 = None
		target = None
		target_1 = None
		port1 = 443
		port2 = 8081
		port3 = 80
		
		# get existing certificate

		alb = elbv2.ApplicationLoadBalancer(self, 
											f'{prefix}-alb', 
											load_balancer_name=prefix, 
											vpc_subnets=ec2.SubnetSelection(subnet_type=ec2.SubnetType.PUBLIC),
											vpc=vpc, idle_timeout=Duration.seconds(600), internet_facing=True)
		certificate_arn = f'arn:aws:acm:{self.region}:{self.account}:certificate/bcf3f163-8178-4b73-b993-c7958dfa0412'
		certificate = acm.Certificate.from_certificate_arn(
			self, "cert", certificate_arn=certificate_arn)
		
		https_listener_2 = alb.add_listener(
		"http_listener", port=port3, protocol=elbv2.ApplicationProtocol.HTTP)  

		https_listener_2.add_action("AdminRedirectActionDefault",
		action=elbv2.ListenerAction.redirect(
			host="#{host}",
			path="/#{path}",
			permanent=True,
			port="443",
			protocol="HTTPS",
			query="#{query}",     
		)
		)    

		https_listener_2.add_action("AdminRedirectActionHTTP",
		action=elbv2.ListenerAction.redirect(
			host=f"{environment}.p2u.kr",
			path="/user-login",
			permanent=True,
			port="443",
			protocol="HTTPS",
			query="#{query}",
			
		),
		conditions=[
			elbv2.ListenerCondition.host_headers([f"{environment}-admin.p2u.kr"])
		],
		priority=1
		)
	  

		
			
		# define healthcheck
		health_check = elbv2.HealthCheck(
			path="/",
			port="traffic-port",
			interval=Duration.seconds(30),
			timeout=Duration.seconds(10),
			healthy_threshold_count=3,
			unhealthy_threshold_count=10,
			healthy_http_codes="200"
		)

		# create https listener
		https_listener = alb.add_listener(
			"HttpsListener", port=port1, protocol=elbv2.ApplicationProtocol.HTTPS, certificates=[certificate])
		
		host = None
		origin = None
		host = "p2u.kr"
		origin = "admin.p2u.kr"
		https_listener.add_action("AdminRedirectAction",
			action=elbv2.ListenerAction.redirect(
				host=host,
				path="/user-login",
				permanent=True,
				port="443",
				protocol="HTTPS",
				query="#{query}",
				
			),
			conditions=[
				elbv2.ListenerCondition.host_headers([origin])
			],
			priority=1
		)
		
		https_listener_1 = alb.add_listener(
			"HttpsListener_1", port=port2, protocol=elbv2.ApplicationProtocol.HTTPS, certificates=[certificate])

		
		target = https_listener.add_targets(
			"ECS", 
			port=8000, 
			protocol=elbv2.ApplicationProtocol.HTTP,
			targets=[service],
			health_check=health_check,
			stickiness_cookie_duration=Duration.days(1),
			)
		
		target_1 = https_listener_1.add_targets(
			"ECS-target-1", 
			port=8000, 
			protocol=elbv2.ApplicationProtocol.HTTP, 
			targets=[service], 
			health_check=health_check,
			stickiness_cookie_duration=Duration.days(1)
			)


		return alb, https_listener, target, target_1

	
	def create_code_deploy_deployment_group(self, service: cdk.aws_ecs.FargateService, alb: elbv2.ApplicationLoadBalancer, listener: elbv2.ApplicationListener, targetgroup: elbv2.ApplicationTargetGroup, targetgroup_1: elbv2.ApplicationTargetGroup, prefix: str):
				
		deployment_application = codedeploy.EcsApplication(self, "my-application-1", application_name=f'{prefix}-ecs')
		deployment_group = codedeploy.EcsDeploymentGroup(self, "BlueGreenDG-1",
			service=service,
			application=deployment_application,
			blue_green_deployment_config=codedeploy.EcsBlueGreenDeploymentConfig(
				blue_target_group=targetgroup,
				green_target_group=targetgroup_1,
				listener=listener,
			),
		)

		return deployment_group
